@@ -1,8 +1,8 @@
 """Provides a selection model to handle selection."""
+from __future__ import division, absolute_import, unicode_literals
 
 import collections
 
-from cola.compat import set
 from cola.observable import Observable
 from cola.decorators import memoize
 
