@@ -1,6 +1,8 @@
 # Copyright (c) 2008 David Aguilar
 """This module provides miscellaneous Qt utility functions.
 """
+from __future__ import division, absolute_import, unicode_literals
+
 import os
 import re
 
@@ -12,14 +14,13 @@ from PyQt4.QtCore import SIGNAL
 from cola import core
 from cola import gitcfg
 from cola import utils
-from cola import settings
 from cola import resources
-from cola.compat import set
 from cola.decorators import memoize
 from cola.i18n import N_
 from cola.interaction import Interaction
 from cola.models.prefs import FONTDIFF
 from cola.widgets import defs
+from cola.compat import ustr
 
 
 def connect_action(action, fn):
@@ -48,7 +49,7 @@ def prompt(msg, title=None, text=''):
         title = msg
     result = QtGui.QInputDialog.getText(active_window(), msg, title,
                                         QtGui.QLineEdit.Normal, text)
-    return (unicode(result[0]), result[1])
+    return (ustr(result[0]), result[1])
 
 
 def create_listwidget_item(text, filename):
@@ -59,13 +60,18 @@ def create_listwidget_item(text, filename):
     return item
 
 
-def create_treewidget_item(text, filename):
+class TreeWidgetItem(QtGui.QTreeWidgetItem):
+
+    def __init__(self, text, filename, exists):
+        QtGui.QTreeWidgetItem.__init__(self)
+        self.exists = exists
+        self.setIcon(0, cached_icon_from_path(filename))
+        self.setText(0, text)
+
+
+def create_treewidget_item(text, filename, exists=True):
     """Creates a QTreeWidgetItem with text and the icon at filename."""
-    icon = cached_icon_from_path(filename)
-    item = QtGui.QTreeWidgetItem()
-    item.setIcon(0, icon)
-    item.setText(0, text)
-    return item
+    return TreeWidgetItem(text, filename, exists)
 
 
 @memoize
@@ -78,9 +84,10 @@ def confirm(title, text, informative_text, ok_text,
     """Confirm that an action should take place"""
     if icon is None:
         icon = ok_icon()
-    elif icon and isinstance(icon, basestring):
+    elif icon and isinstance(icon, ustr):
         icon = QtGui.QIcon(icon)
     msgbox = QtGui.QMessageBox(active_window())
+    msgbox.setWindowModality(Qt.WindowModal)
     msgbox.setWindowTitle(title)
     msgbox.setText(text)
     msgbox.setInformativeText(informative_text)
@@ -95,13 +102,35 @@ def confirm(title, text, informative_text, ok_text,
     return msgbox.clickedButton() == ok
 
 
+class ResizeableMessageBox(QtGui.QMessageBox):
+
+    def __init__(self, parent):
+        QtGui.QMessageBox.__init__(self, parent)
+        self.setMouseTracking(True)
+        self.setSizeGripEnabled(True)
+
+    def event(self, event):
+        res = QtGui.QMessageBox.event(self, event)
+        event_type = event.type()
+        if (event_type == QtCore.QEvent.MouseMove or
+                event_type == QtCore.QEvent.MouseButtonPress):
+            maxi = QtCore.QSize(1024*4, 1024*4)
+            self.setMaximumSize(maxi)
+            text = self.findChild(QtGui.QTextEdit)
+            if text is not None:
+                expand = QtGui.QSizePolicy.Expanding
+                text.setSizePolicy(QtGui.QSizePolicy(expand, expand))
+                text.setMaximumSize(maxi)
+        return res
+
+
 def critical(title, message=None, details=None):
     """Show a warning with the provided title and message."""
     if message is None:
         message = title
-    mbox = QtGui.QMessageBox(active_window())
+    mbox = ResizeableMessageBox(active_window())
     mbox.setWindowTitle(title)
-    mbox.setTextFormat(QtCore.Qt.PlainText)
+    mbox.setTextFormat(Qt.PlainText)
     mbox.setText(message)
     mbox.setIcon(QtGui.QMessageBox.Critical)
     mbox.setStandardButtons(QtGui.QMessageBox.Close)
@@ -119,8 +148,8 @@ def information(title, message=None, details=None, informative_text=None):
     mbox.setStandardButtons(QtGui.QMessageBox.Close)
     mbox.setDefaultButton(QtGui.QMessageBox.Close)
     mbox.setWindowTitle(title)
-    mbox.setWindowModality(QtCore.Qt.WindowModal)
-    mbox.setTextFormat(QtCore.Qt.PlainText)
+    mbox.setWindowModality(Qt.WindowModal)
+    mbox.setTextFormat(Qt.PlainText)
     mbox.setText(message)
     if informative_text:
         mbox.setInformativeText(informative_text)
@@ -129,7 +158,7 @@ def information(title, message=None, details=None, informative_text=None):
     # Render git.svg into a 1-inch wide pixmap
     pixmap = QtGui.QPixmap(resources.icon('git.svg'))
     xres = pixmap.physicalDpiX()
-    pixmap = pixmap.scaledToHeight(xres, QtCore.Qt.SmoothTransformation)
+    pixmap = pixmap.scaledToHeight(xres, Qt.SmoothTransformation)
     mbox.setIconPixmap(pixmap)
     mbox.exec_()
 
@@ -155,7 +184,7 @@ def selected_treeitem(tree_widget):
     selected = False
     item = tree_widget.currentItem()
     if item:
-        id_number = item.data(0, QtCore.Qt.UserRole).toInt()[0]
+        id_number = item.data(0, Qt.UserRole).toInt()[0]
         selected = True
     return(id_number, selected)
 
@@ -185,7 +214,7 @@ def selection_list(listwidget, items):
 def tree_selection(treeitem, items):
     """Returns model items that correspond to selected widget indices"""
     itemcount = treeitem.childCount()
-    widgetitems = [ treeitem.child(idx) for idx in range(itemcount) ]
+    widgetitems = [treeitem.child(idx) for idx in range(itemcount)]
     selected = []
     for item, widgetitem in zip(items[:len(widgetitems)], widgetitems):
         if widgetitem.isSelected():
@@ -194,6 +223,18 @@ def tree_selection(treeitem, items):
     return selected
 
 
+def tree_selection_items(item):
+    """Returns selected widget items"""
+    count = item.childCount()
+    childitems = [item.child(idx) for idx in range(count)]
+    selected = []
+    for child in childitems:
+        if child.isSelected():
+            selected.append(child)
+
+    return selected
+
+
 def selected_item(list_widget, items):
     """Returns the selected item in a QListWidget."""
     widget_items = list_widget.selectedItems()
@@ -220,10 +261,16 @@ def selected_items(list_widget, items):
     return selection
 
 
-def open_dialog(title, filename=None):
+def open_file(title, directory=None):
     """Creates an Open File dialog and returns a filename."""
-    return unicode(QtGui.QFileDialog
-                        .getOpenFileName(active_window(), title, filename))
+    return ustr(QtGui.QFileDialog
+                        .getOpenFileName(active_window(), title, directory))
+
+
+def open_files(title, directory=None, filter=None):
+    """Creates an Open File dialog and returns a list of filenames."""
+    return (QtGui.QFileDialog
+            .getOpenFileNames(active_window(), title, directory, filter))
 
 
 def opendir_dialog(title, path):
@@ -231,14 +278,14 @@ def opendir_dialog(title, path):
 
     flags = (QtGui.QFileDialog.ShowDirsOnly |
              QtGui.QFileDialog.DontResolveSymlinks)
-    return unicode(QtGui.QFileDialog
+    return ustr(QtGui.QFileDialog
                         .getExistingDirectory(active_window(),
                                               title, path, flags))
 
 
 def save_as(filename, title='Save As...'):
     """Creates a Save File dialog and returns a filename."""
-    return unicode(QtGui.QFileDialog
+    return ustr(QtGui.QFileDialog
                         .getSaveFileName(active_window(), title, filename))
 
 
@@ -271,7 +318,6 @@ def _add_action(widget, text, fn, connect, *shortcuts):
     action = QtGui.QAction(text, widget)
     connect(action, fn)
     if shortcuts:
-        shortcuts = list(set(shortcuts))
         action.setShortcuts(shortcuts)
         action.setShortcutContext(Qt.WidgetWithChildrenShortcut)
         widget.addAction(action)
@@ -300,16 +346,18 @@ def set_items(widget, items):
 
 def icon_file(filename, staged=False, untracked=False):
     """Returns a file path representing a corresponding file path."""
+    exists = True
     if staged:
-        if core.exists(filename):
+        exists = core.exists(filename)
+        if exists:
             ifile = resources.icon('staged-item.png')
         else:
             ifile = resources.icon('removed.png')
     elif untracked:
         ifile = resources.icon('untracked.png')
     else:
-        ifile = utils.file_icon(filename)
-    return ifile
+        (ifile, exists) = utils.file_icon(filename)
+    return (ifile, exists)
 
 
 def icon_for_file(filename, staged=False, untracked=False):
@@ -323,10 +371,12 @@ def create_treeitem(filename, staged=False, untracked=False, check=True):
     for adding to a QListWidget.  "staged" and "untracked"
     controls whether to use the appropriate icons."""
     if check:
-        ifile = icon_file(filename, staged=staged, untracked=untracked)
+        (ifile, exists) = icon_file(filename,
+                                    staged=staged, untracked=untracked)
     else:
+        exists = True
         ifile = resources.icon('staged.png')
-    return create_treewidget_item(filename, ifile)
+    return create_treewidget_item(filename, ifile, exists=exists)
 
 
 def update_file_icons(widget, items, staged=True,
@@ -383,6 +433,14 @@ def help_icon():
     return cached_icon(QtGui.QStyle.SP_DialogHelpButton)
 
 
+def add_icon():
+    return icon('add.svg')
+
+
+def remove_icon():
+    return icon('remove.svg')
+
+
 def open_file_icon():
     return icon('open.svg')
 
@@ -436,38 +494,9 @@ def center_on_screen(widget):
     """Move widget to the center of the default screen"""
     desktop = QtGui.QApplication.instance().desktop()
     rect = desktop.screenGeometry(QtGui.QCursor().pos())
-    cy = rect.height()/2
-    cx = rect.width()/2
-    widget.move(cx - widget.width()/2, cy - widget.height()/2)
-
-
-def save_state(widget, handler=None):
-    if handler is None:
-        handler = settings.Settings()
-    if gitcfg.instance().get('cola.savewindowsettings', True):
-        handler.save_gui_state(widget)
-
-
-def export_window_state(widget, state, version):
-    # Save the window state
-    windowstate = widget.saveState(version)
-    state['windowstate'] = unicode(windowstate.toBase64().data())
-    return state
-
-
-def apply_window_state(widget, state, version):
-    # Restore the dockwidget, etc. window state
-    try:
-        windowstate = state['windowstate']
-        return widget.restoreState(QtCore.QByteArray.fromBase64(str(windowstate)),
-                                   version)
-    except KeyError:
-        return False
-
-
-def apply_state(widget):
-    state = settings.Settings().get_gui_state(widget)
-    return bool(state) and widget.apply_state(state)
+    cy = rect.height()//2
+    cx = rect.width()//2
+    widget.move(cx - widget.width()//2, cy - widget.height()//2)
 
 
 @memoize
@@ -501,7 +530,7 @@ def diff_font_str():
     font_str = gitcfg.instance().get(FONTDIFF)
     if font_str is None:
         font = default_monospace_font()
-        font_str = unicode(font.toString())
+        font_str = ustr(font.toString())
     return font_str
 
 
@@ -527,16 +556,36 @@ def create_button(text='', layout=None, tooltip=None, icon=None):
     return button
 
 
-def create_action_button(tooltip, icon):
+def create_action_button(tooltip=None, icon=None):
     button = QtGui.QPushButton()
-    button.setCursor(QtCore.Qt.PointingHandCursor)
-    button.setFlat(True)
-    button.setIcon(icon)
     button.setFixedSize(QtCore.QSize(16, 16))
-    button.setToolTip(tooltip)
+    button.setCursor(Qt.PointingHandCursor)
+    button.setFlat(True)
+    if tooltip is not None:
+        button.setToolTip(tooltip)
+    if icon is not None:
+        pixmap = icon.pixmap(QtCore.QSize(16, 16))
+        button.setIcon(QtGui.QIcon(pixmap))
     return button
 
 
+def hide_button_menu_indicator(button):
+    cls = type(button)
+    name = cls.__name__
+    stylesheet = """
+        %(name)s::menu-indicator {
+            image: none;
+        }
+    """
+    if name == 'QPushButton':
+        stylesheet += """
+            %(name)s {
+                border-style: none;
+            }
+        """
+    button.setStyleSheet(stylesheet % {'name': name})
+
+
 class DockTitleBarWidget(QtGui.QWidget):
 
     def __init__(self, parent, title, stretch=True):
@@ -547,13 +596,13 @@ class DockTitleBarWidget(QtGui.QWidget):
         label.setFont(font)
         label.setText(title)
 
-        self.setCursor(QtCore.Qt.OpenHandCursor)
+        self.setCursor(Qt.OpenHandCursor)
 
         self.close_button = create_action_button(
-                N_('Close'), titlebar_close_icon())
+                tooltip=N_('Close'), icon=titlebar_close_icon())
 
         self.toggle_button = create_action_button(
-                N_('Detach'), titlebar_normal_icon())
+                tooltip=N_('Detach'), icon=titlebar_normal_icon())
 
         self.corner_layout = QtGui.QHBoxLayout()
         self.corner_layout.setMargin(defs.no_margin)
@@ -621,12 +670,12 @@ def create_toolbutton(text=None, layout=None, tooltip=None, icon=None):
     button.setAutoRaise(True)
     button.setAutoFillBackground(True)
     button.setCursor(Qt.PointingHandCursor)
-    if icon:
+    if icon is not None:
         button.setIcon(icon)
-    if text:
+    if text is not None:
         button.setText(text)
         button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-    if tooltip:
+    if tooltip is not None:
         button.setToolTip(tooltip)
     if layout is not None:
         layout.addWidget(button)
@@ -717,7 +766,7 @@ class GenericSyntaxHighligher(QtGui.QSyntaxHighlighter):
     def highlightBlock(self, qstr):
         if not self.enabled:
             return
-        ascii = unicode(qstr)
+        ascii = ustr(qstr)
         if not ascii:
             return
         formats = self.formats(ascii)
@@ -806,7 +855,7 @@ class DiffSyntaxHighlighter(GenericSyntaxHighligher):
                                              diff_head,
                                              diff_head))
         if self.whitespace:
-            self.create_rules('(..*?)(\s+)$', (None, bad_ws))
+            self.create_rules(r'(..*?)(\s+)$', (None, bad_ws))
 
 
 def install():
