@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 import os
 
 from PyQt4 import QtCore
@@ -12,6 +14,7 @@ from cola.git import git
 from cola.git import STDOUT
 from cola.i18n import N_
 from cola.widgets import defs
+from cola.compat import ustr
 
 
 class ExpandableGroupBox(QtGui.QGroupBox):
@@ -73,7 +76,7 @@ class ExpandableGroupBox(QtGui.QGroupBox):
 class GitArchiveDialog(QtGui.QDialog):
 
     @staticmethod
-    def save(ref, shortref, parent=None):
+    def save_hashed_objects(ref, shortref, parent=None):
         dlg = GitArchiveDialog(ref, shortref, parent)
         if dlg.exec_() != dlg.Accepted:
             return None
@@ -81,7 +84,8 @@ class GitArchiveDialog(QtGui.QDialog):
 
     def __init__(self, ref, shortref=None, parent=None):
         QtGui.QDialog.__init__(self, parent)
-        self.setWindowModality(Qt.WindowModal)
+        if parent is not None:
+            self.setWindowModality(Qt.WindowModal)
 
         # input
         self.ref = ref
@@ -218,13 +222,13 @@ class GitArchiveDialog(QtGui.QDialog):
         self.update_filetext_for_format(self.format_combo.currentIndex())
 
     def filetext_changed(self, qstr):
-        self.filename = unicode(qstr)
+        self.filename = ustr(qstr)
         self.save.setEnabled(bool(self.filename))
         prefix = self.strip_exts(os.path.basename(self.filename)) + '/'
         self.prefix_text.setText(prefix)
 
     def prefix_text_changed(self, qstr):
-        self.prefix = unicode(qstr)
+        self.prefix = ustr(qstr)
 
     def strip_exts(self, text):
         for format_string in self.format_strings:
@@ -235,7 +239,7 @@ class GitArchiveDialog(QtGui.QDialog):
 
     def update_filetext_for_format(self, idx):
         self.fmt = self.format_strings[idx]
-        text = self.strip_exts(unicode(self.filetext.text()))
+        text = self.strip_exts(ustr(self.filetext.text()))
         self.filename = '%s.%s' % (text, self.fmt)
         self.filetext.setText(self.filename)
         self.filetext.setFocus()
