@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import Qt
@@ -22,12 +24,14 @@ from cola.qtutils import connect_action_bool
 from cola.qtutils import connect_button
 from cola.qtutils import create_toolbutton
 from cola.qtutils import diff_font
+from cola.qtutils import hide_button_menu_indicator
 from cola.qtutils import options_icon
 from cola.qtutils import save_icon
 from cola.widgets import defs
 from cola.widgets.selectcommits import select_commits
 from cola.widgets.spellcheck import SpellCheckTextEdit
 from cola.widgets.text import HintedLineEdit
+from cola.compat import ustr
 
 
 class CommitMessageEditor(QtGui.QWidget):
@@ -74,6 +78,7 @@ class CommitMessageEditor(QtGui.QWidget):
                                                 tooltip=N_('Actions...'))
         self.actions_button.setMenu(self.actions_menu)
         self.actions_button.setPopupMode(QtGui.QToolButton.InstantPopup)
+        hide_button_menu_indicator(self.actions_button)
 
         self.actions_menu.addAction(self.signoff_action)
         self.actions_menu.addAction(self.commit_action)
@@ -225,7 +230,7 @@ class CommitMessageEditor(QtGui.QWidget):
         "extended description" field.
 
         """
-        value = unicode(value)
+        value = ustr(value)
         if '\n' in value:
             summary, description = value.split('\n', 1)
             description = description.lstrip('\n')
@@ -267,7 +272,7 @@ class CommitMessageEditor(QtGui.QWidget):
             return
 
         # Parse the "summary" and "description" fields
-        umsg = unicode(message)
+        umsg = ustr(message)
         lines = umsg.splitlines()
 
         num_lines = len(lines)
@@ -505,28 +510,15 @@ class CommitSummaryLineEdit(HintedLineEdit):
 
 
 class CommitMessageTextEdit(SpellCheckTextEdit):
+
     def __init__(self, parent=None):
         hint = N_('Extended description...')
         SpellCheckTextEdit.__init__(self, hint, parent)
         self.extra_actions = []
-        self.setMinimumSize(QtCore.QSize(1, 1))
 
         self.action_emit_leave = add_action(self,
                 'Shift Tab', self.emit_leave, 'Shift+tab')
 
-        self.installEventFilter(self)
-
-    def eventFilter(self, obj, event):
-        if event.type() == QtCore.QEvent.FocusIn:
-            height = QtGui.QFontMetrics(self.font()).height() * 3
-            height += defs.spacing * 4
-            self.setMinimumSize(QtCore.QSize(1, height))
-
-        elif event.type() == QtCore.QEvent.FocusOut:
-            self.setMinimumSize(QtCore.QSize(1, 1))
-
-        return False
-
     def contextMenuEvent(self, event):
         menu, spell_menu = self.context_menu()
         if self.extra_actions:
@@ -551,7 +543,7 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
                     self.emit_leave()
                 event.accept()
                 return
-            text_before = unicode(self.toPlainText())[:position]
+            text_before = ustr(self.toPlainText())[:position]
             lines_before = text_before.count('\n')
             if lines_before == 0:
                 # If we're on the first line, but not at the
@@ -568,7 +560,7 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
         elif event.key() == Qt.Key_Down:
             cursor = self.textCursor()
             position = cursor.position()
-            all_text = unicode(self.toPlainText())
+            all_text = ustr(self.toPlainText())
             text_after = all_text[position:]
             lines_after = text_after.count('\n')
             if lines_after == 0:
@@ -584,3 +576,8 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
 
     def emit_leave(self):
         self.emit(SIGNAL('leave()'))
+
+    def setFont(self, font):
+        SpellCheckTextEdit.setFont(self, font)
+        fm = self.fontMetrics()
+        self.setMinimumSize(QtCore.QSize(1, fm.height() * 2))
