@@ -1,4 +1,5 @@
 """Provides dialogs for comparing branches and commits."""
+from __future__ import division, absolute_import, unicode_literals
 
 from PyQt4 import QtCore
 from PyQt4 import QtGui
@@ -11,6 +12,7 @@ from cola.i18n import N_
 from cola.qtutils import connect_button
 from cola.widgets import defs
 from cola.widgets import standard
+from cola.compat import ustr
 
 
 class FileItem(QtGui.QTreeWidgetItem):
@@ -137,12 +139,12 @@ class CompareBranchesDialog(standard.Dialog):
     def selection(self):
         left_item = self.left_list.currentItem()
         if left_item and left_item.isSelected():
-            left_item = unicode(left_item.text())
+            left_item = ustr(left_item.text())
         else:
             left_item = None
         right_item = self.right_list.currentItem()
         if right_item and right_item.isSelected():
-            right_item = unicode(right_item.text())
+            right_item = ustr(right_item.text())
         else:
             right_item = None
         return (left_item, right_item)
@@ -220,10 +222,10 @@ class CompareBranchesDialog(standard.Dialog):
         to reflect the available items.
         """
         if left:
-            which = unicode(self.left_combo.currentText())
+            which = ustr(self.left_combo.currentText())
             widget = self.left_list
         else:
-            which = unicode(self.right_combo.currentText())
+            which = ustr(self.right_combo.currentText())
             widget = self.right_list
         if not which:
             return
