@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 from PyQt4 import QtCore
 from PyQt4 import QtGui
 from PyQt4.QtCore import Qt, SIGNAL
@@ -8,16 +10,21 @@ from cola import gitcmds
 from cola import gravatar
 from cola import qtutils
 from cola.cmds import run
-from cola.compat import set
 from cola.i18n import N_
 from cola.models import main
 from cola.models import selection
+from cola.qtutils import add_action
+from cola.qtutils import create_action_button
+from cola.qtutils import create_menu
 from cola.qtutils import DiffSyntaxHighlighter
+from cola.qtutils import options_icon
 from cola.widgets import defs
 from cola.widgets.text import MonoTextView
+from cola.compat import ustr
 
 
 COMMITS_SELECTED = 'COMMITS_SELECTED'
+FILES_SELECTED = 'FILES_SELECTED'
 
 
 class DiffTextEdit(MonoTextView):
@@ -35,6 +42,42 @@ class DiffEditor(DiffTextEdit):
         DiffTextEdit.__init__(self, parent)
         self.model = model = main.model()
 
+        # "Diff Options" tool menu
+        self.diff_ignore_space_at_eol_action = add_action(self,
+                N_('Ignore changes in whitespace at EOL'),
+                self._update_diff_opts)
+        self.diff_ignore_space_at_eol_action.setCheckable(True)
+
+        self.diff_ignore_space_change_action = add_action(self,
+                N_('Ignore changes in amount of whitespace'),
+                self._update_diff_opts)
+        self.diff_ignore_space_change_action.setCheckable(True)
+
+        self.diff_ignore_all_space_action = add_action(self,
+                N_('Ignore all whitespace'),
+                self._update_diff_opts)
+        self.diff_ignore_all_space_action.setCheckable(True)
+
+        self.diff_function_context_action = add_action(self,
+                N_('Show whole surrounding functions of changes'),
+                self._update_diff_opts)
+        self.diff_function_context_action.setCheckable(True)
+
+        self.diffopts_button = create_action_button(
+                tooltip=N_('Diff Options'), icon=options_icon())
+        self.diffopts_menu = create_menu(N_('Diff Options'),
+                                         self.diffopts_button)
+
+        self.diffopts_menu.addAction(self.diff_ignore_space_at_eol_action)
+        self.diffopts_menu.addAction(self.diff_ignore_space_change_action)
+        self.diffopts_menu.addAction(self.diff_ignore_all_space_action)
+        self.diffopts_menu.addAction(self.diff_function_context_action)
+        self.diffopts_button.setMenu(self.diffopts_menu)
+        qtutils.hide_button_menu_indicator(self.diffopts_button)
+
+        titlebar = parent.titleBarWidget()
+        titlebar.add_corner_widget(self.diffopts_button)
+
         self.action_process_section = qtutils.add_action(self,
                 N_('Process Section'),
                 self.apply_section, Qt.Key_H)
@@ -85,11 +128,24 @@ class DiffEditor(DiffTextEdit):
     def _emit_text(self, text):
         self.emit(SIGNAL('set_text'), text)
 
+    def _update_diff_opts(self):
+        space_at_eol = self.diff_ignore_space_at_eol_action.isChecked()
+        space_change = self.diff_ignore_space_change_action.isChecked()
+        all_space = self.diff_ignore_all_space_action.isChecked()
+        function_context = self.diff_function_context_action.isChecked()
+
+        gitcmds.update_diff_overrides(space_at_eol,
+                                      space_change,
+                                      all_space,
+                                      function_context)
+        self.emit(SIGNAL('diff_options_updated()'))
+
     # Qt overrides
     def contextMenuEvent(self, event):
         """Create the context menu for the diff display."""
         menu = QtGui.QMenu(self)
         s = selection.selection()
+        filename = selection.filename()
 
         if self.model.stageable():
             if s.modified and s.modified[0] in main.model().submodules:
@@ -131,8 +187,13 @@ class DiffEditor(DiffTextEdit):
                 menu.addAction(self.action_unstage_selection)
 
         if self.model.stageable() or self.model.unstageable():
-            menu.addSeparator()
-            menu.addAction(self.launch_editor)
+            # Do not show the "edit" action when the file does not exist.
+            # Untracked files exist by definition.
+            if filename and core.exists(filename):
+                menu.addSeparator()
+                menu.addAction(self.launch_editor)
+
+            # Removed files can still be diffed.
             menu.addAction(self.launch_difftool)
 
         menu.addSeparator()
@@ -186,7 +247,7 @@ class DiffEditor(DiffTextEdit):
             return
 
         offset, selection_text = self.offset_and_selection()
-        old_text = unicode(self.toPlainText())
+        old_text = ustr(self.toPlainText())
 
         DiffTextEdit.setPlainText(self, text)
 
@@ -213,7 +274,7 @@ class DiffEditor(DiffTextEdit):
     def offset_and_selection(self):
         cursor = self.textCursor()
         offset = cursor.position()
-        selection_text = unicode(cursor.selection().toPlainText())
+        selection_text = ustr(cursor.selection().toPlainText())
         return offset, selection_text
 
     # Mutators
@@ -352,6 +413,7 @@ class DiffWidget(QtGui.QWidget):
         self.setLayout(self.main_layout)
 
         notifier.add_observer(COMMITS_SELECTED, self.commits_selected)
+        notifier.add_observer(FILES_SELECTED, self.files_selected)
         self.connect(self.reflector, SIGNAL('diff'), self.diff.setText)
         self.connect(self.reflector, SIGNAL('task_done'), self.task_done)
 
@@ -361,9 +423,9 @@ class DiffWidget(QtGui.QWidget):
         except:
             pass
 
-    def set_diff_sha1(self, sha1):
+    def set_diff_sha1(self, sha1, filename=None):
         self.diff.setText('+++ ' + N_('Loading...'))
-        task = DiffInfoTask(sha1, self.reflector)
+        task = DiffInfoTask(sha1, self.reflector, filename=filename)
         self.tasks.add(task)
         QtCore.QThreadPool.globalInstance().start(task)
 
@@ -371,7 +433,7 @@ class DiffWidget(QtGui.QWidget):
         if len(commits) != 1:
             return
         commit = commits[0]
-        sha1 = commit.sha1
+        self.sha1 = commit.sha1
 
         email = commit.email or ''
         summary = commit.summary or ''
@@ -391,13 +453,19 @@ class DiffWidget(QtGui.QWidget):
         author_template = '%(author)s <%(email)s>' % template_args
         self.author_label.set_template(author_text, author_template)
         self.summary_label.set_text(summary)
-        self.sha1_label.set_text(sha1)
+        self.sha1_label.set_text(self.sha1)
 
-        self.set_diff_sha1(sha1)
+        self.set_diff_sha1(self.sha1)
         self.gravatar_label.set_email(email)
 
+    def files_selected(self, filenames):
+        if not filenames:
+            return
+        self.set_diff_sha1(self.sha1, filenames[0])
+
 
 class TextLabel(QtGui.QLabel):
+
     def __init__(self, parent=None):
         QtGui.QLabel.__init__(self, parent)
         self.setTextInteractionFlags(Qt.TextSelectableByMouse |
@@ -428,7 +496,7 @@ class TextLabel(QtGui.QLabel):
             return
         text = self._metrics.elidedText(self._template,
                                         Qt.ElideRight, width-2)
-        if unicode(text) != self._template:
+        if ustr(text) != self._template:
             self._display = text
 
     # Qt overrides
@@ -446,11 +514,13 @@ class TextLabel(QtGui.QLabel):
 
 
 class DiffInfoTask(QtCore.QRunnable):
-    def __init__(self, sha1, reflector):
+
+    def __init__(self, sha1, reflector, filename=None):
         QtCore.QRunnable.__init__(self)
         self.sha1 = sha1
         self.reflector = reflector
+        self.filename = filename
 
     def run(self):
-        diff = gitcmds.diff_info(self.sha1)
+        diff = gitcmds.diff_info(self.sha1, filename=self.filename)
         self.reflector.emit(SIGNAL('diff'), diff)
