@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 from PyQt4 import QtCore
 from PyQt4 import QtGui
 from PyQt4.QtCore import Qt
@@ -10,21 +12,27 @@ from cola.i18n import N_
 from cola.models import main
 from cola.widgets import defs
 from cola.widgets import text
+from cola.compat import ustr
 
 
-def edit():
-    window = RemoteEditor(qtutils.active_window())
-    window.show()
-    window.raise_()
-    return window
+def remote_editor():
+    view= new_remote_editor(parent=qtutils.active_window())
+    view.show()
+    view.raise_()
+    return view
+
+
+def new_remote_editor(parent=None):
+    return RemoteEditor(parent=parent)
 
 
 class RemoteEditor(QtGui.QDialog):
-    def __init__(self, parent):
+    def __init__(self, parent=None):
         QtGui.QDialog.__init__(self, parent)
 
         self.setWindowTitle(N_('Edit Remotes'))
-        self.setWindowModality(Qt.WindowModal)
+        if parent is not None:
+            self.setWindowModality(Qt.WindowModal)
 
         self.default_hint = N_(''
             'Add and remove remote repositories using the \n'
@@ -48,15 +56,15 @@ class RemoteEditor(QtGui.QDialog):
         self.info_thread = RemoteInfoThread(self)
 
         self.add_btn = QtGui.QToolButton()
-        self.add_btn.setIcon(qtutils.icon('add.svg'))
+        self.add_btn.setIcon(qtutils.add_icon())
         self.add_btn.setToolTip(N_('Add new remote git repository'))
 
         self.refresh_btn = QtGui.QToolButton()
-        self.refresh_btn.setIcon(qtutils.icon('view-refresh.svg'))
+        self.refresh_btn.setIcon(qtutils.reload_icon())
         self.refresh_btn.setToolTip(N_('Refresh'))
 
         self.delete_btn = QtGui.QToolButton()
-        self.delete_btn.setIcon(qtutils.icon('remove.svg'))
+        self.delete_btn.setIcon(qtutils.remove_icon())
         self.delete_btn.setToolTip(N_('Delete remote'))
 
         self.close_btn = QtGui.QPushButton(N_('Close'))
@@ -67,7 +75,7 @@ class RemoteEditor(QtGui.QDialog):
         self._top_layout.addWidget(self.remotes)
         self._top_layout.addWidget(self.info)
         width = self._top_layout.width()
-        self._top_layout.setSizes([width/4, width*3/4])
+        self._top_layout.setSizes([width//4, width*3//4])
 
         self._button_layout = QtGui.QHBoxLayout()
         self._button_layout.addWidget(self.add_btn)
@@ -150,7 +158,7 @@ class RemoteEditor(QtGui.QDialog):
             return
 
         old_name = self.remote_list[idx]
-        new_name = unicode(item.text())
+        new_name = ustr(item.text())
         if new_name == old_name:
             return
         if not new_name:
