@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import Qt
@@ -5,26 +7,26 @@ from PyQt4.QtCore import SIGNAL
 from PyQt4.QtGui import QDockWidget
 
 from cola import core
+from cola import gitcfg
 from cola import qtcompat
-from cola import qtutils
 from cola import settings
 
 
 class WidgetMixin(object):
+    """Mix-in for common utilities and serialization of widget state"""
 
-    # not exported
     def __init__(self, QtClass):
         self.QtClass = QtClass
+        self._apply_state_applied = False
 
-    # Mix-in for standard view operations
     def show(self):
         """Automatically centers dialogs"""
-        if self.parent():
+        if not self._apply_state_applied and self.parent() is not None:
             left = self.parent().x()
             width = self.parent().width()
-            center_x = left + width/2
+            center_x = left + width//2
 
-            x = center_x - self.width()/2
+            x = center_x - self.width()//2
             y = self.parent().y()
 
             self.move(x, y)
@@ -35,6 +37,16 @@ class WidgetMixin(object):
         """Returns the name of the view class"""
         return self.__class__.__name__.lower()
 
+    def save_state(self, handler=None):
+        if handler is None:
+            handler = settings.Settings()
+        if gitcfg.instance().get('cola.savewindowsettings', True):
+            handler.save_gui_state(self)
+
+    def restore_state(self):
+        state = settings.Settings().get_gui_state(self)
+        return bool(state) and self.apply_state(state)
+
     def apply_state(self, state):
         """Imports data for view save/restore"""
         result = True
@@ -51,6 +63,7 @@ class WidgetMixin(object):
                 self.showMaximized()
         except:
             result = False
+        self._apply_state_applied = result
         return result
 
     def export_state(self):
@@ -68,7 +81,7 @@ class WidgetMixin(object):
     def closeEvent(self, event):
         s = settings.Settings()
         s.add_recent(core.getcwd())
-        qtutils.save_state(self, handler=s)
+        self.save_state(handler=s)
         self.QtClass.closeEvent(self, event)
 
 
@@ -79,18 +92,25 @@ class MainWindowMixin(WidgetMixin):
         # Dockwidget options
         self.dockwidgets = []
         self.lock_layout = False
-        qtcompat.set_common_dock_options(self)
         self.widget_version = 0
+        qtcompat.set_common_dock_options(self)
 
     def export_state(self):
         """Exports data for save/restore"""
         state = WidgetMixin.export_state(self)
+        windowstate = self.saveState(self.widget_version)
         state['lock_layout'] = self.lock_layout
-        return qtutils.export_window_state(self, state, self.widget_version)
+        state['windowstate'] = windowstate.toBase64().data().decode('ascii')
+        return state
 
     def apply_state(self, state):
-        WidgetMixin.apply_state(self, state)
-        result = qtutils.apply_window_state(self, state, self.widget_version)
+        result = WidgetMixin.apply_state(self, state)
+        windowstate = state.get('windowstate', None)
+        if windowstate is None:
+            result = False
+        else:
+            result = self.restoreState(QtCore.QByteArray.fromBase64(str(windowstate)),
+                                       self.widget_version) and result
         self.lock_layout = state.get('lock_layout', self.lock_layout)
         self.update_dockwidget_lock_state()
         self.update_dockwidget_tooltips()
@@ -227,23 +247,97 @@ class TreeMixin(object):
         return selected_items[0]
 
 
-def bind_mixin(Mixin, QtClass):
-    """Construct a class which composes the Mixin over the Qt class"""
+class DraggableTreeMixin(TreeMixin):
+    """A tree widget with internal drag+drop reordering of rows"""
 
-    class BoundMixin(Mixin, QtClass):
-        """A concrete class tied to a specific Qt class"""
+    def __init__(self, QtClass):
+        super(DraggableTreeMixin, self).__init__(QtClass)
+        self.setAcceptDrops(True)
+        self.setSelectionMode(self.SingleSelection)
+        self.setDragEnabled(True)
+        self.setDropIndicatorShown(True)
+        self.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
+        self.setSortingEnabled(False)
+        self._inner_drag = False
 
-        def __init__(self, parent=None):
-            QtClass.__init__(self, parent)
-            Mixin.__init__(self, QtClass)
-            self.Mixin = BoundMixin
+    def dragEnterEvent(self, event):
+        """Accept internal drags only"""
+        self.QtClass.dragEnterEvent(self, event)
+        self._inner_drag = event.source() == self
+        if self._inner_drag:
+            event.acceptProposedAction()
+        else:
+            event.ignore()
 
-    return BoundMixin
+    def dragLeaveEvent(self, event):
+        self.QtClass.dragLeaveEvent(self, event)
+        if self._inner_drag:
+            event.accept()
+        else:
+            event.ignore()
+        self._inner_drag = False
+
+    def dropEvent(self, event):
+        """Re-select selected items after an internal move"""
+        if not self._inner_drag:
+            event.ignore()
+            return
+        clicked_items = self.selected_items()
+        event.setDropAction(Qt.MoveAction)
+        self.QtClass.dropEvent(self, event)
+
+        if clicked_items:
+            self.clearSelection()
+            for item in clicked_items:
+                self.setItemSelected(item, True)
+        self._inner_drag = False
+        event.accept() # must be called after dropEvent()
+
+    def mousePressEvent(self, event):
+        """Clear the selection when a mouse click hits no item"""
+        clicked_item = self.itemAt(event.pos())
+        if clicked_item is None:
+            self.clearSelection()
+        return self.QtClass.mousePressEvent(self, event)
 
 
-Widget = bind_mixin(WidgetMixin, QtGui.QWidget)
-Dialog = bind_mixin(WidgetMixin, QtGui.QDialog)
-MainWindow = bind_mixin(MainWindowMixin, QtGui.QMainWindow)
+class Widget(WidgetMixin, QtGui.QWidget):
 
-TreeView = bind_mixin(TreeMixin, QtGui.QTreeView)
-TreeWidget = bind_mixin(TreeMixin, QtGui.QTreeWidget)
+    def __init__(self, parent=None):
+        QtGui.QWidget.__init__(self, parent)
+        WidgetMixin.__init__(self, QtGui.QWidget)
+
+
+class Dialog(WidgetMixin, QtGui.QDialog):
+
+    def __init__(self, parent=None):
+        QtGui.QDialog.__init__(self, parent)
+        WidgetMixin.__init__(self, QtGui.QDialog)
+
+
+class MainWindow(MainWindowMixin, QtGui.QMainWindow):
+
+    def __init__(self, parent=None):
+        QtGui.QMainWindow.__init__(self, parent)
+        MainWindowMixin.__init__(self, QtGui.QMainWindow)
+
+
+class TreeView(TreeMixin, QtGui.QTreeView):
+
+    def __init__(self, parent=None):
+        QtGui.QTreeView.__init__(self, parent)
+        TreeMixin.__init__(self, QtGui.QTreeView)
+
+
+class TreeWidget(TreeMixin, QtGui.QTreeWidget):
+
+    def __init__(self, parent=None):
+        QtGui.QTreeWidget.__init__(self, parent)
+        TreeMixin.__init__(self, QtGui.QTreeWidget)
+
+
+class DraggableTreeWidget(DraggableTreeMixin, QtGui.QTreeWidget):
+
+    def __init__(self, parent=None):
+        QtGui.QTreeWidget.__init__(self, parent)
+        DraggableTreeMixin.__init__(self, QtGui.QTreeWidget)
