@@ -1,4 +1,5 @@
 """Provides the StashView dialog."""
+from __future__ import division, absolute_import, unicode_literals
 
 from PyQt4 import QtCore
 from PyQt4 import QtGui
@@ -30,15 +31,15 @@ def stash():
 class StashView(Dialog):
     def __init__(self, model, parent=None):
         Dialog.__init__(self, parent=parent)
-        self.setAttribute(QtCore.Qt.WA_MacMetalStyle)
         self.model = model
         self.stashes = []
         self.revids = []
         self.names = []
 
-        self.setWindowModality(QtCore.Qt.WindowModal)
         self.setWindowTitle(N_('Stash'))
-        if parent:
+        self.setAttribute(QtCore.Qt.WA_MacMetalStyle)
+        if parent is not None:
+            self.setWindowModality(QtCore.Qt.WindowModal)
             self.resize(parent.width(), 420)
         else:
             self.resize(700, 420)
@@ -95,7 +96,7 @@ class StashView(Dialog):
         self.main_layt.addLayout(self.btn_layt)
         self.setLayout(self.main_layt)
 
-        self.splitter.setSizes([self.width()/3, self.width()*2/3])
+        self.splitter.setSizes([self.width()//3, self.width()*2//3])
 
         self.update_from_model()
         self.update_actions()
